/*******************************************************************************

  ioaccess.c: IO access code for Lanner Platfomr Digital IO program

  Lanner Platform Miscellaneous Utility
  Copyright(c) 2010 - 2015 Lanner Electronics Inc.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer,
     without modification.
  2. Redistributions in binary form must reproduce at minimum a disclaimer
     similar to the "NO WARRANTY" disclaimer below ("Disclaimer") and any
     redistribution must be conditioned upon including a substantially
     similar Disclaimer requirement for further binary redistribution.
  3. Neither the names of the above-listed copyright holders nor the names
     of any contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  Alternatively, this software may be distributed under the terms of the
  GNU General Public License ("GPL") version 2 as published by the Free
  Software Foundation.

  NO WARRANTY
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTIBILITY
  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  THE POSSIBILITY OF SUCH DAMAGES.

*******************************************************************************/

#include "../include/config.h"


#ifdef DJGPP

/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
/* For DOS DJGPP */
#include <dos.h>
#include <inlines/pc.h>

#else //DJGPP
/* For Linux */


#ifdef DIRECT_IO_ACCESS
/* For Linux direct io access code */
/* standard include file */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if defined(LINUX_ENV)
#include <sys/io.h>
#endif

#if defined(FreeBSD_ENV)
#include <machine/cpufunc.h>
#endif

#include <time.h>
#include <stdint.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#define delay(x) usleep(x)
#endif


#ifdef MODULE

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/io.h>
#include <linux/delay.h>

#undef delay
#define delay(x) mdelay(x)
#undef fprintf
#define fprintf(S, A)  printk(A)

#endif //MODULE

#ifdef KLD_MODULE

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/errno.h>

#include <machine/bus.h>
#include <machine/resource.h>

#endif

#endif

/* local include file */
#include "../include/ioaccess.h"

#if (defined(MODULE) || defined(DIRECT_IO_ACCESS) || defined(KLD_MODULE))


/*
 *------------------------------------------------------------------------------
 * LEB-7233 Version V0.1
 *
 * The IO interface for Digital DIO is connected to fintek SIO F81866.
 * Platform provide 2 digital input and 2 digital output. 
 * GPIO0,1 as input function, GPIO2,3 as output function
 * Refer to Fintek 81865 datasheet for details.
 * The truth table is defined as below:
 * DIO GPIO pins as follows:
 *          IN          OUT
 * DIO      GP50        GP54
 * DIO      GP51        GP55
 * DIO      GP52        GP56
 * DIO      GP53        GP57
 *-----------------------------------------------------------------------------------
 */

/*
 * Device Depend Definition : F81866 SIO
*/
#define INDEX_PORT              0x4E
#define DATA_PORT               0x4F
#define BANK_REG                    0x07

#define N_SIO_LDN6_GPIO                 0x06
#define N_SIO_LDN7_WDT                  0x07
#define N_SIO_LDNA_GPIO                 0x0A
#define N_SIO_LDN10_UART1               0x10
#define N_SIO_LDN11_UART2               0x11
#define N_SIO_LDN12_UART3               0x12
#define N_SIO_LDN13_UART4               0x13
#define N_SIO_LDN14_UART5               0x14
#define N_SIO_LDN15_UART6               0x15
#define R_SIO_CID1                      0x20
#define N_SIO_FINTEK_CID1                   0x10
#define R_SIO_CID2                      0x21
#define N_SIO_FINTEK_CID2                   0x10
#define R_SIO_VID1                      0x23
#define N_SIO_FINTEK_VID1                   0x19
#define R_SIO_VID2                      0x24
#define N_SIO_FINTEK_VID2                   0x34
#define N_SIO_FINTEK_VID                    0x3419
#define R_SIO_CLK_SEL                   0x26
#define R_SIO_PORT_SEL                  0x27
#define B_SIO_CLK_TUNE_PROG                 0x01
#define B_SIO_GPIO_PROG_SEL                 0x0C
#define R_SIO_MULTI_FUN_SEL_1_2         0x28
#define R_SIO_MULTI_FUN_SEL_3           0x29
#define R_SIO_GPIO0_EN                  0x2C
#define B_SIO_GPIO00_EN                     0x01
#define B_SIO_GPIO01_EN                     0x02
#define B_SIO_GPIO02_EN                     0x04
#define B_SIO_GPIO03_EN                     0x08
#define B_SIO_GPIO04_EN                     0x10
#define B_SIO_GPIO10_EN                     0x01
#define B_SIO_GPIO11_EN                     0x02
#define B_SIO_GPIO12_EN                     0x04
#define B_SIO_GPIO13_EN                     0x08
#define B_SIO_GPIO15_EN                     0x20
#define B_SIO_GPIO16_EN                     0x40
#define B_SIO_GPIO17_EN                     0x80
#define R_SIO_LDN_ACTIVE                0x30

#define R_SIO_GPIO0_IO_SEL              0xF0
#define R_SIO_GPIO0_OUT_LVL             0xF1
#define R_SIO_GPIO0_IN_LVL              0xF2
#define R_SIO_GPIO0_DRIVE_SEL           0xF3
#define B_SIO_GPIO_00                       0x01
#define B_SIO_GPIO_01                       0x02
#define B_SIO_GPIO_02                       0x04
#define B_SIO_GPIO_03                       0x08
#define B_SIO_GPIO_04                       0x10
#define B_SIO_GPIO_05                       0x20
#define B_SIO_GPIO_06                       0x40
#define B_SIO_GPIO_07                       0x80
#define R_SIO_GPIO5_IO_SEL              0xA0
#define R_SIO_GPIO5_OUT_LVL             0xA1
#define R_SIO_GPIO5_IN_LVL              0xA2
#define R_SIO_GPIO5_DRIVE_SEL           0xA3
#define B_SIO_GPIO_50                       0x01
#define B_SIO_GPIO_51                       0x02
#define B_SIO_GPIO_52                       0x04
#define B_SIO_GPIO_53                       0x08
#define B_SIO_GPIO_54                       0x10
#define B_SIO_GPIO_55                       0x20
#define B_SIO_GPIO_56                       0x40
#define B_SIO_GPIO_57                       0x80
#define R_SIO_WDT_CONFIG                0xF5
#define B_SIO_WDT_TRIG_LVL                  0x04
#define B_SIO_WDT_TIMER_UNIT                0x08
#define B_SIO_WDT_TRIG_MODE                 0x10
#define B_SIO_WDT_EN                        0x20
#define B_SIO_WDT_TIMEOUT_STA               0x40
#define R_SIO_WDT_TIMER                 0xF6


void enter_F81866_config(void)
{
    outportb(INDEX_PORT, 0x87); // Must Do It Twice
    outportb(INDEX_PORT, 0x87);
    return;
}

void exit_F81866_config(void)
{
    outportb(INDEX_PORT, 0xAA);
    return;
}

unsigned char read_F81866_reg(int LDN, int reg)
{
    unsigned char tmp = 0;
    
    enter_F81866_config();
    outportb(INDEX_PORT, BANK_REG); // LDN Register
    outportb(DATA_PORT, LDN); // Select LDNx
    outportb(INDEX_PORT, reg); // Select Register
    tmp = inportb( DATA_PORT); // Read Register
    exit_F81866_config();
    return tmp;
}


void write_F81866_reg(int LDN, int reg, int value)
{
    enter_F81866_config();
    outportb(INDEX_PORT, BANK_REG); // LDN Register
    outportb(DATA_PORT, LDN); // Select LDNx
    outportb(INDEX_PORT, reg); // Select Register
    outportb(DATA_PORT, value); // Write Register
    exit_F81866_config();
    return;
}

void dio_gpio_init(void)
{
    unsigned char tmp;
    
    /* Enable GPIO 5x function */
    // CR27h BIT3-2: 00
    // CR28h Bit6=0, BIT1-0: 00
    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_PORT_SEL);
    tmp &= ~0x0C;
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_PORT_SEL, tmp);

    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_MULTI_FUN_SEL_1_2);
    tmp = (tmp & (~0x03)) | 0x40;   // Set Bit6 and clear Bit 1, 0
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_MULTI_FUN_SEL_1_2, tmp);
    
    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_LDN_ACTIVE);
    tmp |= 0x01;
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_LDN_ACTIVE, tmp);
    
    /* set GPIO50~53 as input function, GPIO54~57 as output */
    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_IO_SEL);
    tmp &= ~(B_SIO_GPIO_50 | B_SIO_GPIO_51 | B_SIO_GPIO_52 | B_SIO_GPIO_53);
    tmp |= (B_SIO_GPIO_54 | B_SIO_GPIO_55 | B_SIO_GPIO_56 | B_SIO_GPIO_57);
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_IO_SEL, tmp);
        
    /* set GPIO50~57 as Open Drain */
    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_DRIVE_SEL);
    tmp &= ~(B_SIO_GPIO_50 | B_SIO_GPIO_51 | B_SIO_GPIO_52 | B_SIO_GPIO_53);
    tmp &= ~(B_SIO_GPIO_54 | B_SIO_GPIO_55 | B_SIO_GPIO_56 | B_SIO_GPIO_57);
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_DRIVE_SEL, tmp);
    
    return;
}

void dio_set_output(unsigned char out_value)
{
    unsigned char tmp;
    
    tmp = read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_OUT_LVL);
    tmp &= ~(B_SIO_GPIO_54 | B_SIO_GPIO_55 | B_SIO_GPIO_56 | B_SIO_GPIO_57);
    tmp |= (out_value << 4);
    write_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_OUT_LVL, tmp);
    
    return;
}

unsigned char dio_get_input(void)
{
    unsigned char tmp,tmp1;
    
    tmp=read_F81866_reg(N_SIO_LDN6_GPIO, R_SIO_GPIO5_IN_LVL);
    return tmp;
}

#endif
